unit Barcode;

interface

uses
  Graphics;

  { Tehdn viivakoodi. H = sivukorkeus pikselein }
procedure PrintCode128(const S: String; Canv: TCanvas; H, E: Integer);

implementation

uses
  Windows, SysUtils, Classes;

const
  BarPat: array[0..106] of PChar = (
  '212222',
  '222122',
  '222221',
  '121223',
  '121322',
  '131222',
  '122213',
  '122312',
  '132212',
  '221213',
  '221312',
  '231212',
  '112232',
  '122132',
  '122231',
  '113222',
  '123122',
  '123221',
  '223211',
  '221132',
  '221231',
  '213212',
  '223112',
  '312131',
  '311222',
  '321122',
  '321221',
  '312212',
  '322112',
  '322211',
  '212123',
  '212321',
  '232121',
  '111323',
  '131123',
  '131321',
  '112313',
  '132113',
  '132311',
  '211313',
  '231113',
  '231311',
  '112133',
  '112331',
  '132131',
  '113123',
  '113321',
  '133121',
  '313121',
  '211331',
  '231131',
  '213113',
  '213311',
  '213131',
  '311123',
  '311321',
  '331121',
  '312113',
  '312311',
  '332111',
  '314111',
  '221411',
  '431111',
  '111224',
  '111422',
  '121124',
  '121421',
  '141122',
  '141221',
  '112214',
  '112412',
  '122114',
  '122411',
  '142112',
  '142211',
  '241211',
  '221114',
  '413111',
  '241112',
  '134111',
  '111242',
  '121142',
  '121241',
  '114212',
  '124112',
  '124211',
  '411212',
  '421112',
  '421211',
  '212141',
  '214121',
  '412121',
  '111143',
  '111341',
  '131141',
  '114113',
  '114311',
  '411113',
  '411311',
  '113141',
  '114131',
  '311141',
  '411131',
  '211412',     // 103 start code A
  '211214',     // 104 start code B
  '211232',     // 105 start code C
  '2331112');   // 106 stop

  { viivakoodin pituus sovitetaan maks.- ja minimimittojen vlille
    niin, ett viivanpaksuus menee tasan pikselien kanssa }
  MaxPit = 105;  // viivakoodin maks.pituus/mm
  MinPit = 70;   // minimipituus/mm
  Kork = 12;     // viivakoodinkorkeus/mm  (10-12,7)
  Alaraja = 0;   // koodin etisyys sivun alareunasta/mm (max. 5)
  { viivakoodi tehdn kytettvn alueen keskelle }
  Vas = 0;       // kytettvn alueen vasen reuna
  Oik = 150;     // kytettvn alueen oikea reuna/mm


  { XP:ss ei toiminut alloleva, tss vanhempi versio, joka toimi }
procedure PrintCodeXP(const S: String; Canv: TCanvas; H: Integer);
var
  Ix, Iy, Iz, Iw, Ib, Ic, Id, Ie, Y1, Y2: Integer;
begin
  with Canv do
  begin
    Ix := Length(S) div 2; // merkkien mr
    Iy := MulDiv(MaxPit * 10, Font.PixelsPerInch, ((Ix + 3) * 11 + 2) * 254);   // pixels/line
    Iz := MulDiv( Iy * 254, (Ix + 3) * 11 + 2, Font.PixelsPerInch * 10);     // total width
    if Iz < MinPit then Exit;    // ei voi tulostaa
    Iw := MulDiv((Oik - Vas - Iz) * 5, Font.PixelsPerInch, 254);                // vas. marg. pixelein
    Pen.Width := 1;
    Pen.Color := clBlack;
    Brush.Color := clBlack;
    Y2 := H - MulDiv(AlaRaja * 10, Font.PixelsPerInch, 254);
    Y1 := Y2 - MulDiv(Kork * 10, Font.PixelsPerInch, 254);
    Iz := 0;
    Ib := 105;  // start code C
    Ic := Ib;   // tarkistussumma
    // piirretn viivat
    repeat
      for Id := 0 to 5 do
      begin
        Ie := Iy * (Ord(BarPat[Ib][Id]) and 15);
        if Id and 1 = 0
        then Rectangle(Iw, Y1, Iw + Ie - 1, Y2);
        Inc(Iw, Ie);
      end;
      Inc(Iz);
      if Iz <= Ix then
      begin
        Ib := StrToInt(Copy(S, 2*Iz - 1, 2));
        Inc(Ic, Iz * Ib);    // tark.summaa
      end
      else if Iz = (Ix + 1)
      then Ib := Ic mod 103   // tark.merkki
      else Ib := 106;                               // stop-merkki
    until Iz > Ix + 2;
    Rectangle(Iw, Y1, Iw + 2 * Iy - 1, Y2);  // stop-merkin viimeinen viiva
    Brush.Color := clWhite;
  end;
end;

  { Tehdn viivakoodi. H = sivukorkeus pikselein,
   pdf-canvasin takia E=1, printerill E=0 }
procedure PrintCode128(const S: String; Canv: TCanvas; H, E: Integer);
var
  Ix, Iy, Iz, Iw, Iv, Ih, Ib, Ic, Id, Ie: Integer;
  Bm: Graphics.TBitmap;
begin
  if E = 2 then 
  begin
    PrintCodeXP(S, Canv, H);
    Exit;
  end;
  Ix := Length(S) div 2; // merkkien mr
  Iy := MulDiv(MaxPit * 10, Canv.Font.PixelsPerInch, ((Ix + 3) * 11 + 2) * 254);   // pixels/line
  Iw := Iy * (Ix + 3) * 11 + 2;                            // viivakoodin leveys pikselein
  if MulDiv(254, Iw, Canv.Font.PixelsPerInch * 10) < MinPit then Exit;  // leveys ei riit
  Ih := MulDiv(Kork * 10, Canv.Font.PixelsPerInch, 254);   // viivakoodin korkeus pikselein
  Bm := Graphics.TBitmap.Create;
  Bm.Width := Iw + 2 * Iy;
  Bm.Height := Ih;
  with Bm.Canvas do
  begin
    Pen.Width := 1;
    Pen.Color := clBlack;
    Brush.Color := clBlack;
    Iv := 0; // vasen aloituskohta
    Iz := 0;
    Ib := 105;  // start code C
    Ic := Ib;   // tarkistussumma
    // piirretn viivat
    repeat
      for Id := 0 to 5 do
      begin
        Ie := Iy * (Ord(BarPat[Ib][Id]) and 15);
        if Id and 1 = 0
        then FillRect(Rect(Iv, 0, Iv + Ie, Ih));
        Inc(Iv, Ie);
      end;
      Inc(Iz);
      if Iz <= Ix then
      begin
        Ib := StrToInt(Copy(S, 2*Iz - 1, 2));
        Inc(Ic, Iz * Ib);    // tark.summaa
      end
      else if Iz = (Ix + 1)
      then Ib := Ic mod 103   // tark.merkki
      else Ib := 106;                               // stop-merkki
    until Iz > Ix + 2;
    FillRect(Rect(Iv, 0, Iv + 2 * Iy, Ih));  // stop-merkin viimeinen viiva
  end;
  Iv := (MulDiv((Oik - Vas) * 10, Canv.Font.PixelsPerInch, 254) - Iw) div 2;   // vas. marg. pikselein
  Ih := H - MulDiv(AlaRaja * 10, Canv.Font.PixelsPerInch, 254) - Ih;           // korkeuspaikka pikselein
  Canv.Draw(Iv, Ih, Bm);
  Bm.Free;
end;



(*  { Tehdn viivakoodi. H = sivukorkeus pikselein,
   pdf-canvasin takia E=1, printerill E=0 }
procedure PrintCode128(const S: String; Canv: TCanvas; H, E: Integer);
var
  Ix, Iy, Iz, Iw, Ib, Ic, Id, Ie, Y1, Y2: Integer;
begin
  with Canv do
  begin
    Ix := Length(S) div 2; // merkkien mr
    Iy := MulDiv(MaxPit * 10, Font.PixelsPerInch, ((Ix + 3) * 11 + 2) * 254);   // pixels/line
    Iz := MulDiv( Iy * 254, (Ix + 3) * 11 + 2, Font.PixelsPerInch * 10);     // total width
    if Iz < MinPit then Exit;    // ei voi tulostaa
    Iw := MulDiv((Oik - Vas - Iz) * 5, Font.PixelsPerInch, 254);                // vas. marg. pixelein
    Pen.Width := 1;
    Pen.Color := clBlack;
    Brush.Color := clBlack;
    Y2 := H - MulDiv(AlaRaja * 10, Font.PixelsPerInch, 254);
    Y1 := Y2 - MulDiv(Kork * 10, Font.PixelsPerInch, 254);
    Iz := 0;
    Ib := 105;  // start code C
    Ic := Ib;   // tarkistussumma
    // piirretn viivat
    repeat
      for Id := 0 to 5 do
      begin
        Ie := Iy * (Ord(BarPat[Ib][Id]) and 15);
        if Id and 1 = 0
        then FillRect(Rect(Iw, Y1, Iw + Ie + E, Y2));
//        then Rectangle(Iw, Y1, Iw + Ie - 1, Y2);
        Inc(Iw, Ie);
      end;
      Inc(Iz);
      if Iz <= Ix then
      begin
        Ib := StrToInt(Copy(S, 2*Iz - 1, 2));
        Inc(Ic, Iz * Ib);    // tark.summaa
      end
      else if Iz = (Ix + 1)
      then Ib := Ic mod 103   // tark.merkki
      else Ib := 106;                               // stop-merkki
    until Iz > Ix + 2;
    FillRect(Rect(Iw, Y1, Iw + 2 * Iy + E, Y2));  // stop-merkin viimeinen viiva
//    Rectangle(Iw, Y1, Iw + 2 * Iy - 1, Y2);  // stop-merkin viimeinen viiva
    Brush.Color := clWhite;
  end;
end;
*)

end.
