{
  TRolePDF
  Dynamically imported
}

unit pdfhdr2;

interface

uses
  Windows, Classes, Graphics, ShellAPI;

type
  TPageSize = (psCustom, psA4, psLetter, psLegal);
  TPageOrientation = (poPortrait, poLandscape);

  TRolePDF = class(TComponent)
  private
    FLicense: TStrings;
    FFileName: string;
    FAuthor: string;
    FTitle: string;
    FSubject: string;
    FKeywords: string;
    FCreator: string;
    FPageWidth: integer;
    FPageHeight: integer;
    FAutoLaunch: boolean;
    FPageSize: TPageSize;
    FOrientation: TPageOrientation;
    FCanvas: TCanvas;
    function GetCanvas: TCanvas;
    function GetPageCount: integer;
    function GetPageWidth: integer;
    function GetPageHeight: integer;
    function DeviceCaps: integer;
    procedure SetPageSize(Value: TPageSize);
    procedure SetOrientation(Value: TPageOrientation);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BeginDoc;
    procedure NewPage;
    procedure EndDoc;
    function AddOutline(Title: string; ParentID, PageNo, TopPos: integer): integer;
    property Canvas: TCanvas read GetCanvas;
  published
    property FileName: string read FFileName write FFileName;
    property Author: string read FAuthor write FAuthor;
    property Title: string read FTitle write FTitle;
    property Subject: string read FSubject write FSubject;
    property Keywords: string read FKeywords write FKeywords;
    property Creator: string read FCreator write FCreator;
    property PageCount: integer read GetPageCount;
    property PageSize: TPageSize read FPageSize write SetPageSize;
    property PageWidth: integer read GetPageWidth write FPageWidth;
    property PageHeight: integer read GetPageHeight write FPageHeight;
    property Orientation: TPageOrientation read FOrientation write SetOrientation;
    property AutoLaunch: boolean read FAutoLaunch write FAutoLaunch;
    property License: TStrings read FLicense write FLicense;
  end;

var
  PdfCreated: Boolean = False;

implementation

var
  RolePDFBeginDoc: procedure(License, FileName, Title, Author, Subject, Keywords,
    Creator: PChar; PageWidth, PageHeight: integer); stdcall;
  RolePDFNewPage: procedure; stdcall;
  RolePDFEndDoc: procedure; stdcall;
  RolePDFGetDC: function: HDC; stdcall;
  RolePDFPageCount: function: integer; stdcall;
  RolePDFPageWidth: function: integer; stdcall;
  RolePDFPageHeight: function: integer; stdcall;
  RolePDFAddOutline: function(Title: PChar; ParentID, PageNo, Top: integer):
    integer; stdcall;
  DllHandle: THandle;

constructor TRolePDF.Create(AOwner: TComponent);
begin
  PdfCreated := False;
  DllHandle := LoadLibrary('pdf.dll');
  if DllHandle <> 0 then
  begin
    @RolePDFBeginDoc := GetProcAddress(DllHandle, 'RolePDFBeginDoc');
    @RolePDFNewPage := GetProcAddress(DllHandle, 'RolePDFNewPage');
    @RolePDFEndDoc := GetProcAddress(DllHandle, 'RolePDFEndDoc');
    @RolePDFGetDC := GetProcAddress(DllHandle, 'RolePDFGetDC');
    @RolePDFPageCount := GetProcAddress(DllHandle, 'RolePDFPageCount');
    @RolePDFPageWidth := GetProcAddress(DllHandle, 'RolePDFPageWidth');
    @RolePDFPageHeight := GetProcAddress(DllHandle, 'RolePDFPageHeight');
    @RolePDFAddOutline := GetProcAddress(DllHandle, 'RolePDFAddOutline');
    PdfCreated := (@RolePDFBeginDoc <> nil) and (@RolePDFNewPage <> nil) and
      (@RolePDFEndDoc <> nil) and (@RolePDFGetDC <> nil) and
      (@RolePDFPageCount <> nil) and (@RolePDFPageWidth <> nil) and
      (@RolePDFPageHeight <> nil) and (@RolePDFAddOutline <> nil);
    if not PdfCreated then FreeLibrary(DllHandle);
  end;
  if PdfCreated then
  begin
    inherited Create(AOwner);
    FLicense := TStringList.Create;
    FCanvas := TCanvas.Create;
    PageSize := psA4;
  end;
end;

destructor TRolePDF.Destroy;
begin
  if PdfCreated then
  begin
    FLicense.Free;
    FCanvas.Free;
    FreeLibrary(DllHandle);
    PdfCreated := False;
  end;
  inherited Destroy;
end;

procedure TRolePDF.SetOrientation(Value: TPageOrientation);

  procedure swap;
  var
    n: integer;
  begin
    n := FPageWidth;
    FPageWidth := FPageHeight;
    FPageHeight := n;
  end;

begin
  FOrientation := Value;
  if FOrientation = poPortrait then begin
    if FPageWidth > FPageHeight then swap;
  end
  else begin
    if FPageWidth < FPageHeight then swap;
  end;
end;

procedure TRolePDF.SetPageSize(Value: TPageSize);
const
  PixelsPerInch = 72;
begin
  FPageSize := Value;
  case Value of
    psA4: begin PageWidth := round(8.26 * PixelsPerInch); PageHeight := round(11.69 * PixelsPerInch); end;
    psLetter: begin PageWidth := round(8.5 * PixelsPerInch); PageHeight := round(11 * PixelsPerInch); end;
    psLegal: begin PageWidth := round(8.5 * PixelsPerInch); PageHeight := round(14 * PixelsPerInch); end;
  end;
end;

procedure TRolePDF.BeginDoc;
var
  i: integer;
  s: string;
begin
  s := '';
  for i := 0 to License.Count - 1 do s := s + License[i];
  RolePDFBeginDoc(PChar(s), PChar(FFileName), PChar(FTitle), PChar(FAuthor),
    PChar(FSubject), PChar(FKeywords), PChar(FCreator), FPageWidth, FPageHeight);
end;

procedure TRolePDF.NewPage;
begin
  RolePDFNewPage;
end;

procedure TRolePDF.EndDoc;
begin
  RolePDFEndDoc;
  if FAutoLaunch then ShellExecute(0, 'open', PChar(FFileName), nil, nil, SW_SHOW);
end;

function TRolePDF.GetCanvas: TCanvas;
begin
  FCanvas.Handle := RolePDFGetDC;
  result := FCanvas;
end;

function TRolePDF.DeviceCaps: integer;
var
  DC: HDC;
begin
  DC := GetDC(0);
  result := GetDeviceCaps(DC, LOGPIXELSX);
  ReleaseDC(0, DC);
end;

function TRolePDF.GetPageCount: integer;
begin
  result := RolePDFPageCount;
end;

function TRolePDF.GetPageWidth: integer;
begin
  result := RolePDFPageWidth;
  if result = 0 then Result := MulDiv(FPageWidth, DeviceCaps, 72);
end;

function TRolePDF.GetPageHeight: integer;
begin
  result := RolePDFPageHeight;
  if result = 0 then Result := MulDiv(FPageHeight, DeviceCaps, 72);
end;

function TRolePDF.AddOutline(Title: string; ParentID, PageNo, TopPos: integer): integer;
begin
  result := RolePDFAddOutline(PChar(Title), ParentID, PageNo, TopPos);
end;

end.
